program DiscardCorrelations
  ! Discard Correlations based on too small or too big complex-amplitudes

  use sep
  implicit none

  integer              :: ii,jj,kk
  integer              :: n1,ntraces,nshots,nkeys!,kept,discarded

  real                 :: bclip,eclip,amplsum

  complex,allocatable  :: datain(:)
  real,allocatable     :: hdrin (:)

  call sep_init()

  call from_param('bclip',bclip,0.)
  call from_param('eclip',eclip,1.)
  
  call from_history('n1',n1,1 )
  call from_history('n2',ntraces,1 )
  call from_history('n3',nshots,1 )
  
  call from_aux('xy_in','n1',nkeys,1 )
  
  allocate(datain(n1),hdrin(nkeys))
  
  !kept=0
  !discarded=0

  do jj=1,ntraces*nshots
	call sreed('in'    ,datain,8*n1   )
	call sreed('xy_in' ,hdrin ,4*nkeys)
	amplsum = sum(cabs(datain))
    	!write(0,*) 'Kept Total:', kept, 'Discarded Total:', discarded ! last one gets cycled and wont print

	if (amplsum.lt.bclip .or. amplsum .gt.eclip) then	! If the trace is mostly "dead"
!write(0,*) 'amplsum:', amplsum, 'jj (trace index):', jj
		hdrin(nkeys) = 1.0 ! Mark trace for killing
    		call srite('xy_out',hdrin ,4*nkeys)
		call srite('out'   ,datain,8*n1   )
		!discarded = discarded + 1
		cycle
	else
		if ( atan2(aimag(datain(1)), real(datain(1))).eq.-0.) then  
			! If the top val of each trace = -0.0 (Should = 0.0)
			hdrin(nkeys) = 1.0 ! Mark trace for killing
    			call srite('xy_out',hdrin ,4*nkeys)
			call srite('out'   ,datain,8*n1   )
			!discarded = discarded + 1
			cycle
		else
    			!kept = kept+1
    			call srite('xy_out',hdrin ,4*nkeys)
			call srite('out'   ,datain,8*n1   )
		end if
    	end if

  end do


  call to_history('n1',nkeys,'xy_out')
  call to_history('n2',ntraces ,'xy_out')
  call to_history('n2',ntraces )

  call sep_close()
  deallocate(datain,hdrin)
  call exit(0)
end program
