
!======================================================
!   PSUEDO CODE FOR EIKONAL.f90
!======================================================


program EIKONAL


!==================================================================

!  If an input trace has a non-zero kill_val, then the eikonal solver
!  will not calculate a velocity for that triangle. Also, if the time-
!  delay value is negative, (they should all be positive) then the 
!  program will also kill that triangle. A good test is that the "number of
!  nonzero samples =" from:
!
!  Window3d < lb-stations-poly-nodes.H n1=1 f1=4 | Attr
!
!  is less than or equal to the same test on 'eikonal.H'
!
!  or number of bad triangles:
!
!  Window3d < time_delay.H@@ n1=1 f1=15 | Attr

!==================================================================

use sep
use eikonal_mod
implicit none

integer					:: n1, n2, ntri, ntraces, nfreq, ff, nnodes, rec_l1, rec_p1, rec_l2, rec_p2, aprox_nshots
integer					:: nkeys, rec_point_key, rec_line_key, it, in, trace, live_tri, dead_tri, aaa, kill_key
integer					:: source_point_key, source_line_key, is, worked

real					:: freq, ofreq, dfreq, td(3), sl, sp
real, dimension(:,:,:), allocatable	:: node_matches
real, dimension(:,:), allocatable	:: triangle, output, all_timedelays, header, tmp, eik_out
real, dimension(:), allocatable		:: timedelay

 call sep_init()

 call from_history("n1",nfreq,1)
 call from_history("n2",ntraces,1)
 call from_history("o1",ofreq,1.0)
 call from_history("d1",dfreq,1.0)

 call from_aux("xylpk","n3",ntri,1)
 call from_aux("xylpk","n2",nnodes,3)
 call from_aux("hdr","n1",nkeys,16)

 call from_param("freq",freq,1.0)
 call from_param("rec_line_key",rec_line_key,3)
 call from_param("rec_point_key",rec_point_key,4)
 call from_param("source_line_key",source_line_key,7)
 call from_param("source_point_key",source_point_key,8)

 call from_param("kill_key",kill_key,16)

 aprox_nshots = int(sqrt(real(ntraces))*1.5)

 allocate( all_timedelays(nfreq,ntraces), timedelay(ntraces), node_matches(3,aprox_nshots,3) )
 allocate( triangle(4,nnodes), output(4,nnodes), tmp(3,nnodes), eik_out(3,nnodes), header(nkeys,ntraces) )
 call sep_read(all_timedelays)
 call sep_read(header,'hdr')

! Grab a single frequency slice from the timedelays
ff = (freq - ofreq)/dfreq + 1
timedelay = all_timedelays(ff,:)

live_tri = 0
dead_tri = 0


! TIME TO PUT ON YOUR BUSINESS SOCKS
do it=1, ntri
	worked=0

	call sep_read(triangle,'xylpk')
	node_matches = -1.0  ! ( We later skip any nodes with negative time delays) cant have an sp or sl that is -1.0 !!
	aaa = 1
	do in=1, nnodes
		rec_l1 = triangle(3,in)
		rec_p1 = triangle(4,in)
		do trace=1, ntraces

			if (timedelay(trace) < 0.0) cycle
			if ( header(kill_key,trace) .ne. 0.0 ) cycle

			rec_l2 = header(rec_line_key,trace)
			rec_p2 = header(rec_point_key,trace)

			! IF YOU FIND A TRACE THAT MATCHES A NODE (AND OCCURED FOR SOURCE 's')... 
			if (rec_l1==rec_l2) then
			if (rec_p1==rec_p2) then
				sl = header(source_line_key ,trace)
				sp = header(source_point_key ,trace)

				
				! CHECK IF LAST NODE FOUND A MATCHING TRACE FOR THE SAME SOURCE 's'
				if (in==1) then
					node_matches(in,aaa,1)=trace
					node_matches(in,aaa,2)=sl
					node_matches(in,aaa,3)=sp
					aaa = aaa + 1
				else if (in==2) then

					do is=1,aaa
						if (node_matches(1,is,2)==sl) then
							if (node_matches(1,is,3)==sp) then
								node_matches(in,is,1) = trace
								node_matches(in,is,2) = sl
								node_matches(in,is,3) = sp
								exit
							end if
						end if
					end do
				else if (in==3) then
					do is=1,aaa		
						if (node_matches(2,is,2)==sl) then
							if (node_matches(2,is,3)==sp) then
								node_matches(in,is,1) = trace
								node_matches(in,is,2) = sl
								node_matches(in,is,3) = sp

								! CALCULATE EIKONAL VELOCITY FOR ALL TRIANGLES 
								! THAT HAVE 3 NODES FOR A COMMON SOURCE 
								td(1) = timedelay(node_matches(1,is,1))
								td(2) = timedelay(node_matches(2,is,1))
								td(3) = timedelay(node_matches(3,is,1))

								! Build (x, y, td) for this node
								tmp(1:2,in) = triangle(1:2,in)	! (x,y) for a node
								tmp(3,:)    = td		! time delay (td) for a node

								! Compute the eikonal plane solution
								call eikonal_solver(tmp,eik_out)
								output(4,:)=0
								output(1:3,:)=eik_out
								call sep_write(output)
								live_tri = live_tri + 1
								worked=1
								exit
							end if
						end if
					end do



				end if
			end if
			end if
		end do
	end do


	if (worked == 0) then
		output = 1 ! (when ALL of output equals one, then the poly program will skip plotting it)
		call sep_write(output)
		!dead_tri = dead_tri + 1
	end if


end do
		
!write(0,*) "dead triangles = ", dead_tri
write(0,*) "live triangles = ", live_tri


! WRAP UP THE LOOSE ENDS
 call to_history("n1",4)
 call to_history("n2",nnodes)
 call to_history("n3",ntri)
 call to_history("o1",1)
 call to_history("d1",1)
 call to_history("label1",'(x, y, velocity [km/s]), kill_val)' )
 call to_history("label2",'Node' )
 call to_history("label3",'Triangle' )

 call sep_close()
deallocate(all_timedelays, timedelay, triangle, output, header)
end program
				
			













 !    Sort3d < time_delay.H nkeys=2 key1=source_line key2=source_point > shot_sorted_H

			
