program EIKONAL_NEW


!==================================================================
!
!  Bin/EIKONAL_NEW.x < sorted_by_shot.H  hdr='sorted_by_shot.H@@'  xylpk='triangle_info.H'  par='${P}/eikonal.p' src_hdr=src_hdr_out.H > $@ 
!
!  Does the eikonal velocity calculation for all triangles and all virtual sources also.
!
!  If the time- delay value is negative, (they should all be positive) then the 
!  program will kill that triangle. A good test is that the "number of
!  nonzero samples =" from:
!
!  Window3d < lb-stations-poly-nodes_ph2.H n1=1 f1=4 | Attr
!
!  is less than or equal to the same test on 'eikonal.H'

!==================================================================

use sep
use eikonal_mod
implicit none

integer					:: n1, n2, ntri, ntraces, nnodes
integer					:: nkeys, rec_point_key, rec_line_key, it, in, trace, found, n3out
integer					:: a, b, ns, nr, maxnr, ir
integer					:: source_line_key, source_point_key, source_x_key, source_y_key, kill_nodes

real					:: vel, azi, src_x, src_y
real					:: tcx, tcy, test_off, sigma
real					:: src_l_old, src_p_old, src_l, src_p, rec_l1, rec_p1, rec_l2, rec_p2

real, dimension(:,:,:), allocatable	:: triangle, output
real, dimension(:,:), allocatable	:: time_hdr, tmp, eik_out, time_hdr_new
real, dimension(:), allocatable		:: timedelay, timedelay_new, td, source_hdr_out

 call sep_init()

 call from_history("n2",ntraces,1)

 call from_aux("xylpk","n3",ntri,1)
 call from_aux("xylpk","n2",nnodes,3)
 call from_aux("hdr","n1",nkeys,16)

 call from_param("rec_line_key",rec_line_key,3)
 call from_param("rec_point_key",rec_point_key,4)
 call from_param("source_x_key",source_x_key,5)
 call from_param("source_y_key",source_y_key,6)
 call from_param("source_line_key",source_line_key,7)
 call from_param("source_point_key",source_point_key,8)
 call from_param("sigma",sigma,0.001)


 allocate( timedelay(ntraces), td(3) )
 allocate( triangle(4,nnodes,ntri), output(6,nnodes,ntri), tmp(3,nnodes), eik_out(3,nnodes), time_hdr(nkeys,ntraces), source_hdr_out(4) )

 timedelay      = 0.0
 td             = 0.0
 triangle       = 0.0
 output         = 0.0
 tmp            = 0.0
 eik_out        = 0.0
 time_hdr       = 0.0


 call sep_read(timedelay)
 call sep_read(time_hdr,'hdr')

 n3out   = 0
 ns      = 1
 nr      = 1
 maxnr   = 1
 kill_nodes = 0

 src_l_old = time_hdr(source_line_key ,1)
 src_p_old = time_hdr(source_point_key,1)

do trace=2,ntraces
	src_l = time_hdr(source_line_key ,trace)
	src_p = time_hdr(source_point_key,trace)
	if ( (abs(src_l - src_l_old)>sigma) .or. (abs(src_p - src_p_old)>sigma)  ) then  ! When source line or point dont match
		ns = ns + 1
		src_l_old = time_hdr(source_line_key ,trace)
		src_p_old = time_hdr(source_point_key,trace)
		nr = 1
	end if
	nr = nr + 1
	if (nr .gt. maxnr) maxnr=nr
end do
allocate( timedelay_new(maxnr), time_hdr_new(nkeys,maxnr) )
timedelay_new = 0.0
time_hdr_new  = 0.0
timedelay_new = -1.0   ! We skip negative time delay values

!------------------------------------------------

! Header History 
  call to_history("n1",4,   'src_hdr')
  call to_history("n2",ns,  'src_hdr')
  call to_history("d1",1.0, 'src_hdr')
  call to_history("d2",1.0, 'src_hdr')
  call to_history("o1",0.0, 'src_hdr')
  call to_history("o2",0.0, 'src_hdr')
  call to_history("label1", '(x, y, src_line, src_point', 'src_hdr')
  call to_history("label2", 'Source Number', 'src_hdr')

! Output History 
 call to_history("n1",6)
 call to_history("n2",nnodes)
 call to_history("n3",ntri)
 call to_history("n4",ns)
 call to_history("o1",0)
 call to_history("o2",0)
 call to_history("o3",0)
 call to_history("o4",0)
 call to_history("d1",1)
 call to_history("d1",1)
 call to_history("d2",1)
 call to_history("d4",1)
 call to_history("label1",'(x, y, velocity [km/s], azimuth, offset, kill_val)' )
 call to_history("label2",'Node' )
 call to_history("label3",'Triangle' )
 call to_history("label4",'Source Number' )

!------------------------------------------------

! Read info for a single triangle
write(0,*) "Reading in the triangles", maxnr,ns
  call sep_read(triangle,'xylpk')

a = 1
b = a
src_l_old = time_hdr(source_line_key ,1)
src_p_old = time_hdr(source_point_key,1)


do trace=2,ntraces
	! Get the traces from a single shot
	src_l = time_hdr(source_line_key ,trace)
	src_p = time_hdr(source_point_key,trace)
	! When source line or point dont match
	if ( (abs(src_l - src_l_old)>sigma) .or. (abs(src_p - src_p_old)>sigma)  .or. trace==ntraces ) then   
		b = trace
		nr = b - a
		time_hdr_new(:,1:nr) = time_hdr(:,a:b-1)
		timedelay_new(1:nr)  = timedelay(a:b-1)

		src_x = time_hdr_new(source_x_key,1)
		src_y = time_hdr_new(source_y_key,1)

		a = b
		output  = 0.0
		source_hdr_out(1) = src_x
		source_hdr_out(2) = src_y
		source_hdr_out(3) = src_l_old
		source_hdr_out(4) = src_p_old
!=====================================================================
		do it=1,ntri
			td = 0.0
			found=0

			! If all nodes on a triangle are considered "live" then ...
			do in=1,nnodes

				! Get node rec_line and rec_point for node
				rec_l1 = triangle(3,in,it)
				rec_p1 = triangle(4,in,it)

				! Find trace that matches node (based on rec_line and rec_point)
				do ir=1, nr
					rec_l2 = time_hdr_new(rec_line_key,ir)
					rec_p2 = time_hdr_new(rec_point_key,ir)

					! But dont even consider trace if timedelay_new(ir) is < 0
                        		if (timedelay_new(ir).le.0.) cycle

					! If a matching trace is found ...
					if (abs(rec_l1-rec_l2)<sigma) then
						if (abs(rec_p1-rec_p2)<sigma) then
							! then grab the time delay value
							td(in) = timedelay_new(ir)
							found = found + 1
							exit
						end if
					end if
				end do

				! Build (x, y, td) for this node
				tmp(1:2,in) = triangle(1:2,in,it)	! (x,y) for a node
				tmp(3,in) = td(in)			! time delay (td) for a node
			end do

			output(1:2,:,it) = triangle(1:2,:,it)

			! Mark triangles if three nodes not found
        		if (found.ne.3) then
				output(6,:,it) = 1
				kill_nodes = kill_nodes + 1
				cycle	
			end if

			! Compute the eikonal plane solution
			call eikonal_solver_va(tmp,vel,azi)

			! Check to see if triangle center is less than min offset away from source
			tcx = sum(triangle(1,:,it))/3.0
			tcy = sum(triangle(2,:,it))/3.0
			test_off = sqrt( (tcx - src_x)**2 + (tcy - src_y)**2 )

			! All triangles that are good get the same outputs below:
			output(3,:,it) = vel
			output(4,:,it) = azi
			output(5,:,it) = test_off   ! abs offset
			output(6,:,it) = 0.0	
!---------------------------------------------------------------------------------
!	write(0,*) "  output = ", output(:,1,it)
!---------------------------------------------------------------------------------
		end do
	call sep_write(output)
	call sep_write(source_hdr_out,'src_hdr')
!=====================================================================
	src_l_old = time_hdr(source_line_key ,trace)
	src_p_old = time_hdr(source_point_key,trace)

	end if
end do

write(0,*) " NUMBER SOURCES FOUND:  ", ns
write(0,*) " NUMBER of triangles:   ", ntri
write(0,*) " NUMBER KILLED for lack of nodes:  ", kill_nodes

 call sep_close()
deallocate(timedelay, timedelay_new, triangle, output, time_hdr, time_hdr_new)
end program


