program ISO_VELOCITY_AVG

use sep
use quantile_mod
implicit none

  integer					:: npar, nnodes, ntri, nsrc, nadded, it, is
  real						:: vel, min_vel, max_vel, off, min_offk, max_offk
  real						:: l1, l2, l3, max_length, quant, low, high
  logical					:: flag
  double precision				:: avg, sum
  real, dimension(:), allocatable 		:: samples
  real, dimension(:,:), allocatable 		:: output
  real, dimension(:,:,:), allocatable 		:: fold
  real, dimension(:,:,:,:), allocatable 	:: input

 call sep_init()

 call from_history("n1", npar,   6)
 call from_history("n2", nnodes, 3)
 call from_history("n3", ntri,   1)
 call from_history("n4", nsrc,   1)

 call from_param("min_vel", min_vel)
 call from_param("max_vel", max_vel)
 call from_param("min_off_killed", min_offk)
 call from_param("max_off_killed", max_offk)
 call from_param("max_length", max_length)
 call from_param("quant", quant)


!============================================================================
! For the average velocity map 
 call to_history("n1",     4 )
 call to_history("n2",nnodes )
 call to_history("n3",ntri   )
 call to_history("n4",     1 )

 call to_history("label1", "(x, y, velocity [km/s], kill_val)" )
 call to_history("label2", "Node" )
 call to_history("label3", "Triangle" )
 call to_history("label4", " " )

! For the fold map 
 call to_history("n1",     4, 'foldout' )
 call to_history("n2",nnodes, 'foldout' )
 call to_history("n3",ntri  , 'foldout' )

 call to_history("label1", '(x, y, fold, kill_val)', 'foldout' )
 call to_history("label2", 'Node', 'foldout' )
 call to_history("label3", 'Triangle', 'foldout' )
 call sep_close()
!============================================================================


 allocate( input(npar,nnodes,ntri,nsrc), output(4,nnodes), fold(4,nnodes,ntri), samples(nsrc) )
 input  = 0.0
 output = 0.0
 fold = 0.0
 call sep_read(input)


 do it=1, ntri

	avg = 0.0
	sum = 0.0
	nadded = 0

	! Assign the default outputs

	output(1:2,:) = input(1:2,:,it,1) ! Assign (x , y)
	output(3,:)   = -1.0
	output(4,:)   = 1.0

	fold(1:2,:,it)   = input(1:2,:,it,1) ! Assign (x , y)
	fold(3,:,it)  = -1.0
	fold(4,:,it)  = 1.0

	! Check to see if triangle has a very long side
	l1 = sqrt(  (input(1,1,it,1) - input(1,2,it,1) )**2 + (input(2,1,it,1) - input(2,2,it,1) )**2 )
	l2 = sqrt(  (input(1,2,it,1) - input(1,3,it,1) )**2 + (input(2,2,it,1) - input(2,3,it,1) )**2 )
	l3 = sqrt(  (input(1,3,it,1) - input(1,1,it,1) )**2 + (input(2,3,it,1) - input(2,1,it,1) )**2 )

	if ( (l1 > max_length) .or. (l2 > max_length) .or. (l3 > max_length) ) then
 		call sep_write(output)
		cycle
	end if

	! Get 'sample' population and quantile values
	samples = input(3,1,it,:)
	low  = quantile(1+nint((   quant)*(nsrc-1)),samples)
	high = quantile(1+nint((1.-quant)*(nsrc-1)),samples)

 	do is=1, nsrc
		off = input(5,1,it,is)
		! Skip triangle if offset is not in range
		if ( ( off > min_offk ) .and. ( off < max_offk ) ) cycle

		! Skip triangle if it is marked for kill
		if ( input(6,1,it,is) > 0 ) cycle

		! Check quantiles for a certain velocity
		vel = input(3,1,it,is)

!write(0,*) "vel = ", vel, "low =", low, "high =", high
   		if (vel.lt.low .and. vel.gt.high) then
			cycle
   		end if

		! Skip triangle if it has very low velocity
		if ( vel < min_vel ) cycle

		! Skip triangle if it has very high velocity
		if ( vel > max_vel ) cycle

		sum = sum + dble(input(3,1,it,is))
		nadded = nadded + 1
	end do

	fold(3,:,it)  = real(nadded)
	if (nadded > 0) then
		avg = (sum / dble(nadded))
		output(3,:)   = real(avg)
		output(4,:)   = 0.0  ! Mark as "live" only if the average is calculated
		fold(4,:,it)     = 0.0  ! Mark as "live" only if the average is calculated
	end if

 ! Write the outputs
 call sep_write(output)
 end do

 if (exist_file('foldout' )) call sep_write(fold , 'foldout' )



end program
