program srPhaseunroll
  use sep
  implicit none
  
  integer          :: n1,n2,n3,n4,n1xy,n2xy
  integer          :: ii,jj,kk,ll,iw
  real             :: d1,o1
  
 ! integer             :: nx,ny,nd,ndxy
 ! real                :: dx,dy
 ! real                :: ox,oy
  
  real   ,allocatable :: trace(:),oldtrace(:)
  real   ,allocatable :: xy_in(:)
  real   ,allocatable :: match(:)
  real                :: minmatch
  real                :: xy_unit
  real                :: old,new
  
  
  integer             :: ic     ! Cycle skip counter
  integer             :: nskips ! Maximum number of skips to check for
  
  real,parameter      :: pi = 3.14159265359

  real                :: c0,f0,f   ! Guessed phase velocity at frequency f
  
  integer             :: sx_key, sy_key, sz_key
  integer             :: rx_key, ry_key, rz_key

  real                :: dist,distance2d,calcphase1,calcphase2

  logical             :: kill,jumped

  
  ! I/O Header
  call sep_init()
  
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  !call from_history('n3',n3,1 )
  !call from_history('n4',n4,1 )
  call from_history('o1',o1,0.)
  call from_history('d1',d1,1.)

  call from_aux('xy','n1',n1xy,1)
  call from_aux('xy','n2',n2xy,1)
  if (n2.ne.n2xy) then; write(0,*) "(n2.ne.n2xy) what is going on man?"; call exit(0); end if

  call from_param('nskips'  ,nskips  ,30  )
  call from_param('minmatch',minmatch,2*pi)
  call from_param('xy_unit',xy_unit,1.)
  call from_param('f0',f0,1.  )  ! m/s
  call from_param('c0',c0,400.)  ! Guessed c0 at f Hz in m/s
  call from_param('rx_key',rx_key,1)
  call from_param('ry_key',ry_key,2)
  call from_param('sx_key',sx_key,3)
  call from_param('sy_key',sy_key,4)
  
  call sep_close()


  allocate(trace(n1  ),oldtrace(n1))
  allocate(xy_in(n1xy))
  allocate(match(nskips+1))
  
  iw=1+nint((f0-o1)/d1)
  
  
  do ll=1,n4
  do kk=1,n3
  do jj=1,n2
    kill=.false.
    trace=0.
    oldtrace=0.
    
    ! For each source
    call sreed('in',trace,4*n1)
    if (sum(abs(trace)).eq.0.) kill=.true.
    oldtrace = trace
    call sreed('xy',xy_in,4*n1xy)
    
    dist = distance2d( (/ xy_in(sx_key),xy_in(sy_key) /) , (/ xy_in(rx_key),xy_in(ry_key) /) )*xy_unit
!    if (jj.lt.10) write(0,*) 'dist:',dist

    ! Test phasematch given a moveout velocity
    f = o1+(iw-1)*d1
    do ic=1,nskips+1
      match(ic) = abs( (trace(iw)-(ic-1)*2*pi) + dist/c0*f*2*pi )
    end do
    ! Find best match and save
    ic = sum(minloc(match))
    trace(iw) = trace(iw)-(ic-1)*2*pi
 !   write(0,*) 'iw',iw,'ic',ic,'f',f,'trace(ii)',trace(iw)
    
    ! Now unroll to lower frequencies
    ic = sum(minloc(match))
    trace(:iw-1) = trace(:iw-1)-(ic-1)*2*pi
    old = trace(iw)
    jumped=.false.
    do ii=iw-1,1,-1
 !     f   = o1+(ii-1)*d1
      if ( abs(trace(ii)).gt.abs(trace(ii+1)) .and. abs(trace(ii)).gt.abs(old) ) then
        trace(:ii) = trace(:ii)+2*pi
        if (jumped) then
          trace(:ii)=0.
          exit
        end if
        jumped=.true.
      else
        jumped=.false.
      end if
      old = trace(ii+1)
    end do
    
    ic = sum(minloc(match))
    ! Now unroll to higher frequencies
    trace(iw+1:) = trace(iw+1:)-(ic-1)*2*pi
    old = trace(iw)
    jumped=.false.
 !   do ii=iw+1,n1-1
 !     f = o1+(ii-1)*d1
      if ( abs(trace(ii)).lt.abs(trace(ii-1)) .and. abs(trace(ii)).lt.abs(old) ) then
        trace(ii:) = trace(ii:)-2*pi
        if (jumped) then
          trace(ii:)=0.
          exit
        end if
        jumped=.true.
      else
        jumped=.false.
      end if
      old = trace(ii-1)
 !   end do
    
    if (kill) trace=0.
    call srite('out',trace,4*n1)
  
  end do
  end do
  end do

  ! Deallocate, close and exit
  deallocate(trace)
  deallocate(xy_in,match)
  call exit(0)

end program



real function distance2d(xy1,xy2)
  real xy1(2),xy2(2)
  distance2d = sqrt( (xy1(1)-xy2(1))**2.  +  (xy1(2)-xy2(2))**2. )
end function


real function calcphase1(phase)
  complex         :: z
  real            :: r,i
  real,parameter  :: pi = 3.14159265359
  z = cexp(cmplx(0.,phase))
  r = atan2(aimag(z),real(z))
  calcphase1 = r
end function

real function calcphase2(phase)
  complex         :: z
  real            :: r,i
  real,parameter  :: pi = 3.14159265359
  z = cexp(cmplx(0.,phase))
  r = atan2(aimag(z),real(z))
  if (r.lt.0.) r=r+2*pi
  calcphase2 = r
end function
