program TAPER_XT

  use sep
  implicit none
	
  integer 		:: nt, it, rt, itrace, t1, t2, tc, nkeys, ntrace, dtrace, otrace, abs_off_key_num
  real	 		:: dt, ot, sigma_t, velocity, val, abs_offset, sigma_x
  real, dimension(:), allocatable	:: trace, trace_out, keys, taper

  call sep_init()

  call from_history('n1',nt)
  call from_history('d1',dt)
  call from_history('o1',ot)
  call from_history('n2',ntrace)
  call from_history('d2',dtrace)
  call from_history('o2',otrace)

  if (int((nt+1)/2) .ne. int((nt)/2)) call erexit("nt is not even")

  call from_aux('header','n1',nkeys,1)

  call from_param('sigma_t',sigma_t)
  call from_param('sigma_x',sigma_x)
  call from_param('velocity',velocity)
  call from_param('abs_off_key_num',abs_off_key_num)

  allocate(trace(nt))
  allocate(keys(nkeys))
  allocate(taper(nt))

!---------------------------------------
  do itrace=1,ntrace
	call sep_read(keys,'header')
	call sep_read(trace)

       	! Determine taper parameters
	abs_offset = keys(abs_off_key_num)
	!write(0,*) " abs_offset = ", abs_offset
	tc = (abs_offset/velocity)/dt
	t1 = tc - sigma_t
	t2 = tc + sigma_t
	taper= 0.0

	!---------------------------------------
	! FIRST Build taper along TIME AXIS
	do it=1, nt
		! First centered tapering along the chosen wavespeed moveout line
		if (it >= t1 .and. it <= t2) then 
			val = cos( (it-tc)*3.14159265358/(2*sigma_t) )**2
			taper(it) = val
		end if	

		! Then along the edges of the time axis near t=0 and t=nt
		if (it<sigma_t) then
			val = cos( (it-sigma_t)*3.14159265358/(2*sigma_t) )**2
			taper(it) = taper(it)*val
		else if (it > (nt-sigma_t)) then
			val = cos( (it+sigma_t-nt)*3.14159265358/(2*sigma_t) )**2
			taper(it) = taper(it)*val
		end if		
	end do

	!---------------------------------------
	! SECOND Build taper along TRACE AXIS
	if (itrace<sigma_x) then
		val = cos( (itrace-sigma_x)*3.14159265358/(2*sigma_x) )**2
		taper(:) = taper(:)*val
	else if (itrace > (ntrace-sigma_x)) then
		val = cos( (itrace+sigma_x-ntrace)*3.14159265358/(2*sigma_x) )**2
		taper(:) = taper(:)*val
	end if

	!---------------------------------------
	! Apply taper
	trace = trace * taper
	!---------------------------------------
	call sep_write(trace)
	call sep_write(taper, "taper.H")
  end do

!---------------------------------------
  call to_history('n1',nt,"taper.H")
  call to_history('n2',ntrace,"taper.H")
  call to_history('d1',dt,"taper.H")
  call to_history('d2',dtrace,"taper.H")
  call to_history('o1',ot, "taper.H")
  call to_history('o2',otrace, "taper.H")
  call to_history('label1','Time', "taper.H")
  call to_history('label2','Trace Number', "taper.H")
!---------------------------------------

  call sep_close()
  deallocate(trace,keys,taper)
  call exit(0)
end program
