program delaunay

!*****************************************************************************
!
!  	THIS CODE DOESNT SEEM TO WORK IF YOU HAVE DUPLICATE POINTS!!!!!!!!!!!


!
!  Discussion:
!
!    DELAUNAY_LMAP_2D computes a Delaunay triangulation under a linear map.
!
!    The dataset is simply a set of points in the plane.
!
!    The linear map is applied using a matrix, which we denote by A.
!    A embodies a linear transformation applied to the data.  
!
!    Thus, given a set of points V1, V2, ..., VN, we will actually
!    be applying a standard Delaunay triangulation to A*V1, A*V2, ..., A*VN.
!
!    In turn, the Delaunay triangulation is an organization of the
!    data into triples, forming a triangulation of the data, with
!    the property that the circumcircle of each triangle never contains
!    another data point.  
!
!    For the linear mapped data, we can interpret this process in one 
!    of two ways:
!
!    * Given V1, V2, ..., VN, we compute A*V1, A*V2, ..., A*VN, and 
!      carry out a standard Delaunay triangulation on that data, so that
!      the linearly mapped Delaunay triangulation has the property that,
!      if Va, Vb, and Vc form a linearly mapped Delaunay triangle, the
!      circumcircle of (A*Va, A*Vb, A*Vc) does not contain the image
!      A*Vd of any other data point Vd.
!
!    * Given V1, V2, ..., VN, we compute a linear mapped Delaunay triangulation
!      which has the property that, for any three points (Va,Vb,Vc)
!      that form a linearly mapped Delaunay triangle, the "circumellipse" which 
!      passes through (Va, Vb, Vc) does not contain any other data point Vd.
!
!    Thanks to Yoshimitsu Asahi for pointing out several errors in a previous
!    version of this program, 08 June 2010.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license. 
!
!  Modified:
!
!    08 June 2010
!
!  Author:
!
!    John Burkardt

  use sep
  use delaunay_mod
  implicit none

  integer :: n1,np,nd,id,it
  real      ( kind = 8 ) matrixr(2,2)
  real      ( kind = 8 ) matrix(2,2)
  real      ( kind = 4 ), allocatable, dimension ( :, : ) :: tabler
  real      ( kind = 8 ), allocatable, dimension ( :, : ) :: table
  integer   ( kind = 4 ), allocatable, dimension ( :, : ) :: triangle_neighbor
  integer   ( kind = 4 ), allocatable, dimension ( :, : ) :: triangle_node

  call sep_init()
  
  ! Get input data as a table
  call from_history('n1',n1,1)
  call from_history('n2',np,1)
  if (n1.ne.2) then
    write(0,*) 'n1.ne.2'
  end if
  allocate ( table(n1,np) , tabler(n1,np))
  call sreed('in',tabler,4*size(tabler))
  table = dble(tabler)
  
  
  ! Get input matrix as a table
  if (exist_file('matrix')) then
    call sreed('in',matrixr,16)
    matrix = dble(matrixr)
  else
    matrix      = 0.
    matrix(1,1) = 1.
    matrix(2,2) = 1.
  end if

!
!  Determine the Delaunay triangulation under the linear map.
!
  allocate ( triangle_node    (3,3*np) )
  allocate ( triangle_neighbor(3,3*np) )

  ! Delaunay Triangulation
  call dtris2_lmap ( np, table, matrix, nd, triangle_node, triangle_neighbor )

  ! Write out data
  call to_history('n1',3 )
  call to_history('n2',nd)
  call to_history('d1',1.)
  call to_history('d2',1.)
  call to_history('o1',1.)
  call to_history('o2',1.)
  call to_history('label1',"3 Node Numbers")
  call to_history('label2',"Triangles")
  call srite('out',real(triangle_node(:,1:nd)),4*3*nd)

  ! Desired Triangle coordinates (for polygon drawing)

  if (exist_file('poly')) then
    call to_history('n1',2 ,'poly')
    call to_history('n2',3 ,'poly')
    call to_history('n3',nd,'poly')
    call to_history('d1',1.,'poly')
    call to_history('d2',1.,'poly')
    call to_history('d3',1.,'poly')
    call to_history('o1',1.,'poly')
    call to_history('o2',1.,'poly')
    call to_history('o3',1.,'poly')
    call to_history('label1',"(x,y)"    ,'poly')
    call to_history('label2',"3 Nodes"  ,'poly')
    call to_history('label3',"Triangles",'poly')
    do id=1,nd
      do it=1,3
        call srite('poly',tabler(:,triangle_node(it,id)),4+4)
      end do
    end do
  end if

  deallocate ( table )
  deallocate ( triangle_neighbor )
  deallocate ( triangle_node )
  call sep_close

  call exit(0);

end




