
module eikonal_mod
implicit none
  contains

  subroutine eikonal_solver(timedelay, vel)
	real, dimension(:,:)	:: timedelay, vel
	integer			:: in
	real			:: dx1, dx2, dy1, dy2, dt1, dt2, i, j, k, grad

	dx1 = timedelay(1, 3) - timedelay(1, 1)
	dx2 = timedelay(1, 2) - timedelay(1, 1)

	dy1 = timedelay(2, 3) - timedelay(2, 1)
	dy2 = timedelay(2, 2) - timedelay(2, 1)
		
	dt1 = timedelay(3, 3) - timedelay(3, 1)  ! Slowness really, not time delay
	dt2 = timedelay(3, 2) - timedelay(3, 1) 

	! Do the cross product
	i = (dy1*dt2 - dt1*dy2)
	j = (dx1*dt2 - dx2*dt1)
	k = (dx1*dy2 - dx2*dy1)

	! Take the magnitude of the vector resulting from cross-product
	
	do in=1,3
		vel(1,in) = timedelay(1,in)	! Grab x values
		vel(2,in) = timedelay(2,in)	! Grab y values
	end do
	vel(3,:) = 1/(sqrt(i**2 + j**2)/abs(k))	! Assign velocity values
  end subroutine






subroutine eikonal_solver_va(timedelay, vel, azi)
	real, dimension(:,:)	:: timedelay
	real			:: vel, azi, dx1, dx2, dy1, dy2, dt1, dt2, i, j, k, grad

	dx1 = timedelay(1, 3) - timedelay(1, 1)
	dx2 = timedelay(1, 2) - timedelay(1, 1)

	dy1 = timedelay(2, 3) - timedelay(2, 1)
	dy2 = timedelay(2, 2) - timedelay(2, 1)
		
	dt1 = timedelay(3, 3) - timedelay(3, 1)  ! Slowness really, not time delay
	dt2 = timedelay(3, 2) - timedelay(3, 1) 

	! Do the cross product

	i = (dx1*dt2 - dx2*dt1)
	j = (dy1*dt2 - dt1*dy2)
	k = (dx1*dy2 - dx2*dy1)


	! Calculate the Velocity	
	vel = abs(k)/(sqrt(i**2 + j**2))	! Assign velocity values

	! Calculate the Azimuth direction
	if ( (i > 0) .and. (j > 0) ) then	! First  quadrant
		azi = atan(j/i)
	else if ( (i < 0) .and. (j > 0) ) then	! Second quadrant
		azi = atan(j/i) + 3.1415926
	else if ( (i < 0) .and. (j < 0) ) then	! Third  quadrant
		azi = atan(j/i) + 3.1415926
	else if ( (i > 0) .and. (j < 0) ) then	! Fourth quadrant
		azi = atan(j/i) + 2*3.1415926
	end if

	azi = azi*180/3.1415926

  end subroutine
!_______________________
!	    |          |
!   Quad2   |   Quad1  |
!-----------------------
!   Quad3   |   Quad4  |
!	    |          |
!-----------------------


end module 
