
#include <string.h>
#include <float.h>
#include <math.h>

#include <rsf.h>
#include <rsfplot.h>

static int n;
static float *t, pclip;
static void getminmax(const float* f, float* min, float* max);
float max_array(float a[], int num_elements);
float min_array(float a[], int num_elements);

int main(int argc, char* argv[])
{
    bool transp;
    int n1, n2, n3, i1, i2, i3, len, ic, nreserve;
    bool framenum;
    float min1, max1, min2, max2, o3, d3, o1, d1, xi, yi, tt, sqsize, scale, minc, maxc;
    float *x, *y, *tmp, *symbolsz=NULL, symsize, xc, yc;    
    float*** data=NULL;
    char *color;
    char* symbol, sym[2]=" ";
    sf_datatype type;
    sf_file in;

    sf_init(argc,argv);
    in = sf_input("in");
    vp_init();

    if (!sf_histint(in,"n1",&n1)) sf_error("No n1= in input");
    if (!sf_histint(in,"n2",&n2)) n2=1;
    if (!sf_histint(in,"n3",&n3)) n3=1;
    if (n2!=3) sf_error("n2!=3 in input");
    
    // n = n1*n2;
    sf_leftsize(in,3);
    if (n3 > 1) {
	if (!sf_histfloat(in,"o3",&o3)) o3=0.;
	if (!sf_histfloat(in,"d3",&d3)) d3=1.;
    }

    x = sf_floatalloc(4);
    y = sf_floatalloc(4);
    t = sf_floatalloc(3);
    
    if (!sf_getfloat("pclip",&pclip)) pclip=100.; /* clip percentile */
    if (!sf_getfloat("sqsize",&sqsize)) sqsize=.1; /* square size */
    if (!sf_getfloat("minc",&minc)) minc= 1.; /* clip fraction */
    if (!sf_getfloat("maxc",&maxc)) maxc=-1.; /* clip fraction */

    type = sf_gettype(in);
    data = sf_floatalloc3(n1,n2,n3);
    
    if (!sf_getbool("transp",&transp)) transp=false;
    /* if y, transpose the display axes */
    /* initialize color table */
    if (NULL == (color = sf_getstring("color"))) color="i";
    /* color scheme (default is i) */
    if (!sf_getint ("nreserve",&nreserve)) nreserve = 8;
    /* reserved colors */
    vp_rascoltab (nreserve, color);
    
    
    // These dont work for some obscure reason, not important
    // read these from param, one given triangle dont define them well anyways.
    
    // Read data
    sf_floatread(data[0][0],n1*n2*n3,in);
    
    
 //   // Scale the color
/*
    min1=min_array(&data[0][2][0],n1);
    for (i3 = 1; i3 < n3; i3++) {
      min2=min_array(&data[i3][2][0],n1);
      if (min2<min1) min1=min2;
   }
   max1=min_array(&data[0][2][0],n1);
    for (i3 = 1; i3 < n3; i3++) {
      max2=max_array(&data[i3][2][0],n1);
      if (max2>max1) max1=max2;
    }
*/


    fprintf(stderr,"minc,maxc: %6.6f  %6.6f\n",minc,maxc);
    scale=256./(maxc-minc);
    fprintf(stderr,"scale: %6.6f\n",scale);
    
    
    if (!sf_getbool("wantframenum",&framenum)) framenum = (bool) (n3 > 1);
    /* if y, display third axis position in the corner */

    // Init vplot
    vp_plot_init(n3);
    vp_stdplot_init (min1,max1,min2,max2, \
                 transp,false,false,true);
    vp_frame_init(in,"blt",false);

    // Do actual v-plot Calls
    ic = 1; // Current color
    for (i3 = 0; i3 < n3; i3++) {
    
    if (framenum) vp_framenum(o3+i3*d3);
    vp_frame();
    
    
    for (i1 = 0; i1 < n1; i1++) {
       
 //      fprintf(stderr,"x: %f  y: %f \n", data[i3][0][i1], data[i3][1][i1]);
       
       // Define square
       x[0] = data[i3][0][i1]+sqsize;
       y[0] = data[i3][1][i1]       ;
       x[1] = data[i3][0][i1]       ;
       y[1] = data[i3][1][i1]+sqsize;
       x[2] = data[i3][0][i1]-sqsize;
       y[2] = data[i3][1][i1]       ;
       x[3] = data[i3][0][i1]       ;
       y[3] = data[i3][1][i1]-sqsize;
       
       // Transpose axis
       if (transp) {tmp=x; x=y; y=tmp;}
       
       // Prepare for next draw
       vp_plot_set (i3);
       
       // Set drawing color
       ic = scale*(data[i3][2][i1]-minc)+256;
       if (ic<256) ic=256;
       if (ic>511) ic=511;
       vp_color(ic);
       
       // Actually draw polygon
       vp_uarea(x, y, 4, 0, 1, 1);
       
       }

       if (i3 > 0) vp_erase (); 
       }
       
    exit(0);
}

static void getminmax(const float* f, float* min, float* max)
{
    int i, m, nc;
    float fmin, fmax, fi, fbig, fsml, fdif;

    m=0;
    fmin=+FLT_MAX;
    fmax=-FLT_MAX;
    for (i=0; i < n; i++) {
	fi = f[i];
	if (finite(fi)) {
	    t[m] = fi;
	    m++;
	    if (fmin > fi) fmin=fi;
	    if (fmax < fi) fmax=fi;
	}
    }

    nc = (1.-0.01*pclip)*m;
    if (nc < 0) nc=0;
    if (nc >= m) nc=m-1;
    
    if (nc > 0 && nc < m-1) {
	fsml = sf_quantile(nc,m,t);
	fbig = sf_quantile(m-nc-1, m,t);
	fdif = fbig - fsml;

	*min = SF_MAX(fmin-fdif/8,fsml-fdif);
	*max = SF_MIN(fmax+fdif/8,fbig+fdif);
    } else {
	*min = fmin;
	*max = fmax;
    }
}


float max_array(float a[], int num_elements)
{
   int i;
   float max=a[0];
   for (i=0; i<num_elements; i++)
   {
	 if (a[i]>max)
	 {
	    max=a[i];
	 }
   }
   return(max);
}

float min_array(float a[], int num_elements)
{
   int i;
   float min=a[0];
   for (i=0; i<num_elements; i++)
   {
	 if (a[i]<min)
	 {
	    min=a[i];
	 }
   }
   return(min);
}




/* 	$Id: graph.c 7107 2011-04-10 02:04:14Z ivlad $	 */
