/* Graph plot.

First written by Sjoerd to write plot Delanay triangles - input is expected to be: 2x3xntrainges
Adapted by Taylor to add color to the triangles - input is expected to be: 3x3xntrainges

Takes: > plot.vpl

Run "sfdoc stdplot" for more parameters.
*/
/*
  Copyright (C) 2004 University of Texas at Austin
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <string.h>
#include <float.h>
#include <math.h>

#include <rsf.h>
#include <rsfplot.h>


static int n;
static float *t, pclip;
static void getminmax(const float* f, float* min, float* max);

int main(int argc, char* argv[])
{
    bool transp;
    int n1, n2, n3, i1, i2, i3, len, ic, nreserve, kill, kill_val;
    float min1, max1, min2, max2, o3, d3, o1, d1, xi, yi, tt, scale, minc, maxc, clip, bias;
    float *x, *y, *tmp, *symbolsz=NULL, symsize, xc, yc;    
    float*** data=NULL;
    char *color;
    char* symbol, sym[2]=" ";
    sf_datatype type;
    sf_file in;

    sf_init(argc,argv);
    in = sf_input("in");
    vp_init();

    if (!sf_histint(in,"n1",&n1)) sf_error("No n1= in input");
    if (!sf_histint(in,"n2",&n2)) n2=1;
    if (!sf_histint(in,"n3",&n3)) n3=1;
    if (n1!=4) sf_error("The input has an n1 not equal to 4. Input should have (x,y,plotting_value, kill_value). ");
    if (n2!=3) sf_error("n2!=3 in input");
    
    n = n1*n2;
    sf_leftsize(in,3);
    if (n3 > 1) {
	if (!sf_histfloat(in,"o3",&o3)) o3=0.;
	if (!sf_histfloat(in,"d3",&d3)) d3=1.;
    }

    x   = sf_floatalloc(3);
    y   = sf_floatalloc(3);


    if (!sf_getfloat("pclip",&pclip)) pclip=100.; /* clip percentile */

    if (!sf_getfloat("clip",&clip)) clip=1.; /* clip fraction */
    if (!sf_getfloat("bias",&bias)) bias=0.; /* clip fraction */

    if (!sf_getfloat("minc",&minc)) minc=bias-clip; /* clip fraction */
    if (!sf_getfloat("maxc",&maxc)) maxc=bias+clip; /* clip fraction */


    fprintf(stderr,"minc,maxc: %6.6f  %6.6f\n",minc,maxc);
    scale=256./(maxc-minc);
    fprintf(stderr,"scale: %6.6f\n",scale);


    type = sf_gettype(in);
    data = sf_floatalloc3(4,3,n3);

    if (!sf_getbool("transp",&transp)) transp=false;
    /* if y, transpose the display axes */

    /* initialize color table */
    if (NULL == (color = sf_getstring("color"))) color="i";
    /* color scheme (default is i) */
    if (!sf_getint ("nreserve",&nreserve)) nreserve = 8;
    /* reserved colors */
    vp_rascoltab (nreserve, color);

    // These dont work for some obscure reason, not important
    // read these from param, one given triangle dont define them well anyways.
    //getminmax(x,&min1,&max1);
    //getminmax(y,&min2,&max2);
    min1=0; max1=1;
    min1=0; max2=1;

    // Transpose
    if (transp) {
	    tt=max1; max1=max2; max2=tt;
	    tt=min1; min1=min2; min2=tt;
       }
    // Read data
    sf_floatread(data[0][0],4*3*n3,in);
    // Init vplot
    vp_plot_init(n3);
    vp_stdplot_init (min1,max1,min2,max2, \
                 transp,false,false,true);
    vp_frame_init(in,"blt",false);
    vp_frame();

    // For each triangle
    ic = 1; // Current color


    kill_val=0;
    for (i3 = 0; i3 < n3; i3++) {

	kill = data[i3][1][3];	// Kill parameter

	for (i2 = 0; i2 < 3; i2++) {
          	x[i2]   = data[i3][i2][0];
          	y[i2]   = data[i3][i2][1];
          	}

       	// Transpose axis
       	if (transp) {tmp=x; x=y; y=tmp;}

       	// Prepare for next draw
	vp_plot_set (i3);

	// Set drawing color
	ic = scale*(data[i3][1][2]-minc)+256;	// Colorscale


	if (ic<256) ic=256;
	if (ic>511) ic=511;
	vp_color(ic);

	if (kill==0) {
		kill_val = kill_val + 1;
//fprintf(stderr,"kill_val :%6.6i\n",kill_val);
		// Actually draw polygon
		vp_uarea(x, y, 3, 0, 1, 1);
		}

   	}
exit(0);
}





static void getminmax(const float* f, float* min, float* max)
{
    int i, m, nc;
    float fmin, fmax, fi, fbig, fsml, fdif;

    m=0;
    fmin=+FLT_MAX;
    fmax=-FLT_MAX;
    for (i=0; i < n; i++) {
	fi = f[i];
	if (finite(fi)) {
	    t[m] = fi;
	    m++;
	    if (fmin > fi) fmin=fi;
	    if (fmax < fi) fmax=fi;
	}
    }

    nc = (1.-0.01*pclip)*m;
    if (nc < 0) nc=0;
    if (nc >= m) nc=m-1;
    
    if (nc > 0 && nc < m-1) {
	fsml = sf_quantile(nc,m,t);
	fbig = sf_quantile(m-nc-1, m,t);
	fdif = fbig - fsml;

	*min = SF_MAX(fmin-fdif/8,fsml-fdif);
	*max = SF_MIN(fmax+fdif/8,fbig+fdif);
    } else {
	*min = fmin;
	*max = fmax;
    }
}


//fprintf(stderr,"kill_val :%6.6i\n",kill_val);

/* 	$Id: graph.c 7107 2011-04-10 02:04:14Z ivlad $	 */
