program srTpick_new2
  ! Program
  ! Sjoerd de Ridder, Stanford, 3/26/2012
  !
  ! true_t:
  ! T: use physical units padd to make symmetric*
  ! F: just stuff through FFT  (default)
  ! fftshift:
  ! T: fftshift data  (default)
  ! F: do not fftshift data
  !
  ! BANDPASS OPTIONS
  ! Taper Specifics (like in srTaper1d)
  ! opt: (taper types 
  ! 0)  'zero'            --  needed: 'nothing' 
  ! 1)  'rect' (default)  --  needed: f1,f2
  ! 2)  'linear'          --  needed: f1,f2
  ! 3)  'pyramid'         --  needed: f1,f2
  ! 4)  'pyramid-band'    --  needed: f1,f2,f3,f4
  ! 5)  'hann'            --  needed: f1,f2
  ! 6)  'hann_half'       --  needed: f1,f2
  ! 7)  'hann-band'       --  needed: f1,f2,f3,f4
  ! 8)  'gaussian'        --  needed: f1,f2,sigma
  ! 9)  'gaussian_half'   --  needed: f1,f2,sigma
  ! 10) 'gaussian_block'  --  needed: f1,f2,f3,f4,sigma
  ! 
  ! f1,f2 default to spanning the axis.
  ! f1,f2,f3,f4 default to splitting the axis three way.
  !
  ! unit (for f1,f2,...):
  ! 1) Hz (default)
  ! 2) Radians
  !
  ! All tapers have the form of a band or lowcut,
  ! Notch tapers and high cut tapers are constructed
  ! with inv=1 option.
  ! Scale the data using parameter scale (default 1.). 
  ! Specify indices instead of physical units using f1_ind,f2_ind,...
  ! Inspect taper by supplying 'taper' as filename-parameter.

  use sep                      ! Main SEP module
  use sep_timers_mod
  implicit none
  
  ! Include for FFTW
  include "fftw3.f"
  include "openmp.inc"
  
  ! Axis units
  integer                         :: n1,nt,n1i,nti,n2,n3,n4,n5,n6,n1h,jt,n2out
  real                            :: d1,d2,dt,o1,o2,ot
  
  integer                         :: nf
  real                            :: of,df

  ! Header keys
  integer                         :: sx_key,sy_key,rx_key,ry_key
  
  ! Memory variables
  real,    allocatable, dimension(:,:)  :: datat ,               datae,                               datab,datb
  complex, allocatable, dimension(:,:)  :: datatc,datafc,datafac,dataac,datafac_or,dataac_or,datae_or,dataf
  complex, allocatable, dimension(:)    :: data_dft1, dafa_dft1
  complex, allocatable, dimension(:)    :: data_dft2, dafa_dft2

  real,    allocatable, dimension(:)    :: t,taper
  real,    allocatable, dimension(:,:)  :: hdrs,values
  real,    allocatable, dimension(:,:)  :: data_pos , data_neg
  real,    allocatable, dimension(:,:)  :: data_pos_or , data_neg_or

  real,    allocatable, dimension(:)  :: pickdiff
  real,    allocatable, dimension(:)  :: tt

  real,    allocatable, dimension(:)  :: rad,ang,wine,winb
  
  
  ! For Bandpass
  ! For Tapers
  integer                         :: f1_ind,f2_ind,f3_ind,f4_ind,hind
  real                            :: f1,    f2,    f3,    f4
  real                            :: scaler
  real                            :: sigma  ! For Gaussian taper
  integer                         :: opt
  logical                         :: inv
  integer                         :: funit    ! 1: Hz      2: Radians
  
  
  ! Counters
  integer                         :: ii,jj,kk
  
  logical                         :: verbose,true_t,fftshift,extend
  
  ! FFTW plans  
  integer*8                       :: fft_for,fft_inv
  
  ! Parameters
  real, parameter                 :: pi=3.1415926535
  ! Input Parameters 
  real                            :: v
  real                            :: tau0,dtau,xy_unit
  integer                         :: en,bp,nn,np,ne
  integer                         :: cnt,dtaui
  
  ! Output numbers
  real, allocatable, dimension(:) :: sn_pos, sn_neg, snr
  real                            :: sig_in,sig_out
  integer, allocatable, dimension(:) :: t0,tind,tindp,tindn
  real                            :: thresh

  real                            :: amplitude,phase
  real                            :: antisymrejection

  real, allocatable, dimension(:) :: peakwidth
  integer                         :: peakup,peaklo
  
  ! For timers
  logical           :: logic
  integer           :: tm
  logic = setup_next_timer("Times",tm)
  
  ! Open I/O headerfiles
  call sep_init()
  call from_param('verbose',verbose,.false.)
  call from_param('verb'   ,verbose,verbose)
  if (verbose) write(0,*) '-------------------------------------------'
  if (verbose) write(0,*) '           PICK ENVELOPE PEAKS             '
  if (verbose) write(0,*) '-------------------------------------------'
  if (verbose) write(0,*) '        i/o looping over traces            '
  call from_param('extend',extend,.false.)  

  call from_param('thresh',thresh,0.)
  call from_param('antisymrejection',antisymrejection,.2)

  include "openmp_init.inc"

  ! Read axis Details
  call from_history("n1",n1,1 )
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )
  call from_history("n4",n4,1 )
  call from_history("n5",n5,1 )
  call from_history("n6",n6,1 )
  call from_history("d1",d1,1.)
  call from_history("d2",d2,1.)
  call from_history("o1",o1,0.)
  call from_history("o2",o2,0.)
  
  call from_param("dt",dt,d1)  ! At what time sampling to interpolate (rounded to integer division of d1)
  jt = maxval( (/ nint(d1/dt) , 1 /) )
  call from_param("jt",jt,jt)  ! Sumsampling integer
  dt=d1/jt
  ! write(0,*) "jt:",jt
  if (floor(real(n1)/2.).ne.ceiling(real(n1)/2.)) call erexit("This would be a lot easier if n1 is divisible by 2")
  
  ! Axis Units
  n1  = n1
  nt  = n1/2    ! nt Means the number of positive time samples
  ! Interpolated Units
  n1i = jt*n1
  nti = n1i/2   ! nt Means the number of positive time samples
  
  allocate(tt(n1i))
  do ii=1,n1i
    tt(ii)=o1+(ii-1)*dt
  end do

 ! write(0,*) n1,nt,n1i,nti
 ! call from_param('sym',sym,.true.)
  
  call from_param(      'v',      v, 350.)
  call from_param('xy_unit',xy_unit,   1.)
  call from_param(   'dtau',   dtau,   1.)
  dtaui = nint(dtau/dt)
  
  ! From header
  if (exist_file('header')) then
    call from_aux("header","n1",n1h,1)
    call from_param( "rx_key" ,rx_key,1)
    call from_param( "ry_key" ,ry_key,2)
    call from_param( "sx_key" ,sx_key,3)
    call from_param( "sy_key" ,sy_key,4)
  end if
  
 ! write(0,*) 'dtaui,dtau,dt'
 ! write(0,*)  dtaui,dtau,dt
  
  !!! Prepare optional Bandpass
  ! Fourier units
  of = 0.
  df = 1./(n1*d1)
  nf = n1/2+1
  if (funit.eq.2) df=2.*pi*df
  
  ! Read input parameters for taper, there is lots of options :-)
  call from_param("opt"  ,  opt,1)
  call from_param('funit',funit,1)
  
  if (opt.lt.0..or.opt.gt.10) opt =  1
  call from_param ("inv"  , inv, .false. )
  call from_param ("sigma",sigma,   1.   )
  
  ! Read Tie points from param
  call from_param ("f1"    ,f1    , of          )
  call from_param ("f1_ind",f1_ind, nint((f1-of)/df)+1)
  
  if (opt.eq.4 .or. opt.eq.7 .or. opt.eq.10) then
  call from_param ("f2"    ,f2    , of+1*nf/3*df)
  call from_param ("f2_ind",f2_ind, nint((f2-of)/df)+1)
  else
  call from_param ("f2"    ,f2    , of+(nf-1)*df)
  call from_param ("f2_ind",f2_ind, nint((f2-of)/df)+1)
  end if

  call from_param ("f3"    ,f3    , of+2*nf/3*df)
  call from_param ("f3_ind",f3_ind, nint((f3-of)/df)+1)
  
  call from_param ("f4"    ,f4    , of+(nf-1)*df)
  call from_param ("f4_ind",f4_ind, nint((f4-of)/df)+1)

  ! Make sure they fall within range
  if (f1_ind.lt.1.) f1_ind=1
  if (f2_ind.lt.1.) f2_ind=1
  if (f3_ind.lt.1.) f3_ind=1
  if (f4_ind.lt.1.) f4_ind=1
  if (f1_ind.gt.nf) f1_ind=nf
  if (f2_ind.gt.nf) f2_ind=nf
  if (f3_ind.gt.nf) f3_ind=nf
  if (f4_ind.gt.nf) f4_ind=nf
  
  if (verbose) then
    if (opt.eq.4 .or. opt.eq.7 .or. opt.eq.10) then
      if (verbose) write(0,"(A,F,X,F,X,F,X,F)") ' taper tie values : ',f1    ,f2    ,f3    ,f4
      if (verbose) write(0,"(A,I3.3,X,I3.3,X,I3.3,X,I3.3)") ' taper tie indices: ',f1_ind,f2_ind,f3_ind,f4_ind
    else
      if (verbose) write(0,"(A,F,X,F)") ' taper tie values : ',f1    ,f2
      if (verbose) write(0,"(A,I,X,I)") ' taper tie indices: ',f1_ind,f2_ind
    end if
  end if
  
  allocate(taper(nf)); taper=0
  select case(opt)

    case(0)
      if (verbose) write(0,*) 'taper: 0.'

    case(1)
      if (verbose) write(0,*) 'taper: rectangle'
      taper(f1_ind:f2_ind)=1.

    case(2)
      if (verbose) write(0,*) 'taper: linear'
      taper(1:f1_ind)=1.
      do ii=f1_ind,f2_ind
        taper(ii)=1.-real(ii-f1_ind)/real(f2_ind-f1_ind)
      end do
      
    case(3)
      if (verbose) write(0,*) 'taper: pyramid'
      hind = nint(real(f2_ind-f1_ind)/2.)
      do ii=f1_ind,f1_ind+hind
        taper(ii)=   real(ii-f1_ind     )/real(f1_ind+hind-f1_ind)
      end do
      do ii=f1_ind+hind,f2_ind
        taper(ii)=1.-real(ii-f1_ind-hind)/real(f2_ind-f1_ind-hind)
      end do
      
    case(4)
      if (verbose) write(0,*) 'taper: pyramid-band'
      hind = nint(real(f2_ind-f1_ind)/2.)
      do ii=f1_ind,f2_ind
        taper(ii)=   real(ii-f1_ind     )/real(f2_ind-f1_ind)
      end do
      taper(f2_ind:f3_ind) = 1.
      do ii=f3_ind,f4_ind
        taper(ii)=1.-real(ii-f3_ind     )/real(f4_ind-f3_ind)
      end do
      
    case(5)
      if (verbose) write(0,*) 'taper: hann'
      do ii=f1_ind,f2_ind
        taper(ii)= .5*(1.-cos(2.*pi*(ii-f1_ind)/real(f2_ind-f1_ind)))
      end do
      
    case(6)
      if (verbose) write(0,*) 'taper: hann-half'
      taper(1:f1_ind) = 1.
      do ii=f1_ind,f2_ind
        taper(ii)= .5*(1.+cos(   pi*(ii-f1_ind)/real(f2_ind-f1_ind)))
      end do
      
    case(7)
      if (verbose) write(0,*) 'taper: hann-band'
      do ii=f1_ind,f2_ind
        taper(ii)= .5*(1.-cos(pi*(ii-f1_ind)/real(f2_ind-f1_ind)))
      end do
      taper(f2_ind:f3_ind) = 1.
      do ii=f3_ind,f4_ind
        taper(ii)= .5*(1.+cos(pi*(ii-f3_ind)/real(f4_ind-f3_ind)))
      end do

    case(8)
      if (verbose) write(0,*) 'taper: gaussian'
      hind = nint(real(f2_ind-f1_ind))
      do ii=f1_ind,f2_ind
        taper(ii)= exp(-(ii-hind)**2./(2*d1*sigma**2.) )
      end do
      
    case(9)
      if (verbose) write(0,*) 'taper: gaussian-half'
      taper(1:f1_ind)=1.
      do ii=f1_ind,f2_ind
        taper(ii)= exp(-((ii-f1_ind)**2.)/(2*d1*sigma**2.) )
      end do
      
    case(10)
      if (verbose) write(0,*) 'taper: gaussian-band'
      do ii=f1_ind,f2_ind
        taper(ii)= exp(-((ii-f2_ind)**2.)/(2*d1*sigma**2.) )
      end do
      taper(f2_ind:f3_ind) = 1.
      do ii=f3_ind,f4_ind
        taper(ii)= exp(-((ii-f3_ind)**2.)/(2*d1*sigma**2.) )
      end do
  
  end select
  
  ! Invert taper to lowpass or notch
  if (inv) taper = 1.-taper

  ! Taper as output
  if (exist_file("taper")) then
    if (verbose) write(0,*) 'write taper to file'
    call to_history("n1", nf ,"taper")
    call to_history("d1", df ,"taper")
    call to_history("o1", of ,"taper")
    call srite("taper",taper,4*nf)
  end if
  
  if (exist_file('env')) then
    call to_history("n1",n1i,'env')
    call to_history("d1",dt ,'env')
    call to_history("d2",d2 ,'env')
    call to_history("o1",o1 ,'env')
    call to_history("o2",o2 ,'env')
    call to_history("n2",n2 ,'env')
    call to_history("n3",n3 ,'env')
    call to_history("n4",n4 ,'env')
    call to_history("n5",n5 ,'env')
    call to_history("n6",n6 ,'env')
  end if
  if (exist_file('env_or')) then
    call to_history("n1",n1i,'env_or')
    call to_history("d1",dt ,'env_or')
    call to_history("d2",d2 ,'env_or')
    call to_history("o1",o1 ,'env_or')
    call to_history("o2",o2 ,'env_or')
    call to_history("n2",n2 ,'env_or')
    call to_history("n3",n3 ,'env_or')
    call to_history("n4",n4 ,'env_or')
    call to_history("n5",n5 ,'env_or')
    call to_history("n6",n6 ,'env_or')
  end if

  if (exist_file('pos')) then
    call to_history("n1",nti,'pos')
    call to_history("d1",dt ,'pos')
    call to_history("d2",d2 ,'pos')
    call to_history("o1",0. ,'pos')
    call to_history("o2",o2 ,'pos')
    call to_history("n2",n2 ,'pos')
    call to_history("n3",n3 ,'pos')
    call to_history("n4",n4 ,'pos')
    call to_history("n5",n5 ,'pos')
    call to_history("n6",n6 ,'pos')
  end if

  if (exist_file('neg')) then
    call to_history("n1",nti,'neg')
    call to_history("d1",dt ,'neg')
    call to_history("d2",d2 ,'neg')
    call to_history("o1",0. ,'neg')
    call to_history("o2",o2 ,'neg')
    call to_history("n2",n2 ,'neg')
    call to_history("n3",n3 ,'neg')
    call to_history("n4",n4 ,'neg')
    call to_history("n5",n5 ,'neg')
    call to_history("n6",n6 ,'neg')
  end if

  if (exist_file('data')) then
    call to_history("n1",n1i,'data')
    call to_history("d1",dt ,'data')
    call to_history("d2",d2 ,'data')
    call to_history("o1",o1 ,'data')
    call to_history("o2",o2 ,'data')
    call to_history("n2",n2 ,'data')
    call to_history("n3",n3 ,'data')
    call to_history("n4",n4 ,'data')
    call to_history("n5",n5 ,'data')
    call to_history("n6",n6 ,'data')
  end if

  ! Allocate memories for trace
  allocate(datat  (n1 ,n2))
  allocate(datatc (n1 ,n2))
  allocate(datafc (n1 ,n2))
  allocate(datafac(n1i,n2))
  allocate(dataac (n1i,n2))
  allocate(datae  (n1i,n2))
  
  allocate(datab  (n1i,n2), datb(n1i,n2))
  allocate(dataf  (n1i,n2))
  
  allocate(hdrs   (n1h,n2))
  
  allocate( data_neg(nti,n2) )
  allocate( data_pos(nti,n2) )
  allocate( data_neg_or(nti,n2) )
  allocate( data_pos_or(nti,n2) )
  allocate( datae_or(n1i,n2) )
  
  allocate(data_dft1(n1 ), dafa_dft1(n1 ))
  allocate(data_dft2(n1i), dafa_dft2(n1i))
  
  allocate(rad(n2),ang(n2),t0(n2),winb(n2),wine(n2))
  allocate(tindn(n2),tindp(n2),tind(n2),sn_pos(n2),sn_neg(n2),snr(n2))
  
  allocate(datafac_or(n1i,n2),dataac_or(n1i,n2))

  allocate(peakwidth(n2))
  
  if (.not.extend) allocate(values(21    ,n2))
  if (     extend) allocate(values(21+n1h,n2))
  
  ! Taper as output
  allocate(pickdiff(n2))
  if (exist_file("pickdiff")) then
    call to_history("n1", n2*n3,"pickdiff")
    call to_history("d1", 1. ,"pickdiff")
    call to_history("o1", 1. ,"pickdiff")
  end if
  
  ! Prepare fftw plans
  call sfftw_plan_dft_1d(fft_for, n1 , data_dft1, dafa_dft1, 1, FFTW_ESTIMATE);
  call sfftw_plan_dft_1d(fft_inv, n1i, dafa_dft2, data_dft2,-1, FFTW_ESTIMATE);
  
  write(0,*) 'testing tt - axis:'
  write(0,*)  'mint:', tt(    1)
  write(0,*)  'maxt:', tt(n1i  )
  write(0,*)   '-dt:', tt(nti  )
  write(0,*)    '0.:', tt(nti+1)
  write(0,*)   '+dt:', tt(nti+2)
  
  n2out=0
  if (verbose) write(0,*) '-------------------------------------------'
!  if (verbose) write(0,*) 'symmetrize:',sym
  !!!! Actual Kernel
  call start_timer_num(tm)
  do kk=1,n3

    if (verbose) write(0,*) 'kk:',kk
    hdrs     = 0.
    datat    = 0.

    dataf    = 0.
    datab    = 0.
    datb     = 0.
    datatc   = 0.
    datafc   = 0.
    datafac  = 0.
    dataac   = 0.
    datae    = 0.
    data_pos = 0.
    data_neg = 0.
    values   = 0.

    pickdiff = 0.

    rad = 0.

    rad = 0.
    ang = 0.
    t0  = 0.
    winb = 0.
    wine = 0.
    
    tindn = 0.
    tindp = 0.

    sn_pos = 0.
    sn_neg = 0.
    snr   = 0.
    
    ! Read the data
    call sreed("header",hdrs ,4*n1h*n2)
    call sreed('in'    ,datat,4*n1 *n2)

    if (mod(jj,n2).eq.0) then
      if (verbose) write(0,"(A,I7.3,A,I7.3)") 'shot: ',kk,' out of ',n3
    end if
    
  !  if (sum(abs(datat)).lt.thresh) then
  !    if (exist_file('env')) call srite("env",datae,4*n1i)
  !    cycle
  !  end if
    
    ! Convert to complex and fftshift    
    datatc(     1:n1/2,:) = cmplx(datat(n1/2+1:n1  ,:),0.)
    datatc(n1/2+1:n1  ,:) = cmplx(datat(     1:n1/2,:),0.)
    
    ! FFT the data
    do jj=1,n2
      data_dft1 = datatc(:,jj)
      call sfftw_execute( fft_for )
      datafc(:,jj) = dafa_dft1
    end do
    
    dataf                    = 0.
    dataf(         1:n1/2,:) = datafc(     1:n1/2,:)
 !   dataf(n1i-n1/2+1:n1i ,:) = datafc(n1/2+1:n1  ,:)

    
    ! Create analytic signal
    datafac                = 0.
    datafac(     1     ,:) =   datafc(     1     ,:)
    datafac(     1:n1/2,:) = 2*datafc(     1:n1/2,:)
    
    ! Bandpass and spectral balancing by taper
!    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(nf,phase,datafac,taper)
    do jj=1,n2
    do ii=1,nf
      datafac_or(ii,jj) = taper(ii)*datafac(ii,jj)
      phase             = atan2(aimag(datafac(ii,jj)),real(datafac(ii,jj)))
      datafac   (ii,jj) = taper(ii)*cexp(cmplx(0,phase))
    end do
    end do
!    !$OMP END PARALLEL DO
    
!    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(nf,phase,datafac,taper)
!    if (exist_file('data')) then
      do jj=1,n2
      do ii=1,nf
        phase             = atan2(aimag(dataf  (ii,jj)),real(dataf  (ii,jj)))
        dataf     (ii,jj) = taper(ii)*cexp(cmplx(0,phase))
      end do
      end do
!    end if
!    !$OMP END PARALLEL DO

!    write(0,*) 'datafac',datafac(:,100)
    
    do jj=1,n2
      dafa_dft2 = datafac(:,jj)
      call sfftw_execute( fft_inv )
      dataac(:,jj) = data_dft2
      
      dafa_dft2 = datafac_or(:,jj)
      call sfftw_execute( fft_inv )
      dataac_or(:,jj) = data_dft2
      
      dafa_dft2 = dataf  (:,jj)
      call sfftw_execute( fft_inv )
      datab    (:,jj) = real(data_dft2)
    end do
    
    ! Absolute value of the analytic signal (envelope) and ifftshift
    datae(      1:n1i/2,:) = abs(dataac(n1i/2+1:n1i  ,:))
    datae(n1i/2+1:n1i  ,:) = abs(dataac(      1:n1i/2,:))
    datae_or(      1:n1i/2,:) = abs(dataac_or(n1i/2+1:n1i  ,:))
    datae_or(n1i/2+1:n1i  ,:) = abs(dataac_or(      1:n1i/2,:))
    datb(      1:n1i/2,:)  = datab(n1i/2+1:n1i  ,:)
    datb(n1i/2+1:n1i  ,:)  = datab(      1:n1i/2,:)
    
    ! Write the picked envelope function
    if (exist_file('env'))    call srite("env"   ,datae   ,4*n1i*n2)
    if (exist_file('env_or')) call srite("env_or",datae_or,4*n1i*n2)
    if (exist_file('data'))   call srite("data"  ,datb    ,4*n1i*n2)
    
    ! Sort data into positve and negative windows
    data_pos(1,:)      =   datae(nti+1,:)
    do ii=1,nti-1
      data_pos(1+ii,:) =   datae(nti+1+ii,:)
    end do
    data_neg(1,:)      =   datae(nti+1,:)
    do ii=1,nti-1
      data_neg(1+ii,:) =   datae(nti+1-ii,:)
    end do
    
    ! Sort data into positve and negative windows
    data_pos_or(1,:)      =   datae_or(nti+1,:)
    do ii=1,nti-1
      data_pos_or(1+ii,:) =   datae_or(nti+1+ii,:)
    end do
    data_neg_or(1,:)      =   datae_or(nti+1,:)
    do ii=1,nti-1
      data_neg_or(1+ii,:) =   datae_or(nti+1-ii,:)
    end do
    
    if (exist_file('neg')) then
      call srite("neg",data_neg,4*size(data_neg))
    end if
    if (exist_file('pos')) then
      call srite("pos",data_pos,4*size(data_pos))
    end if
    
!    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(n2,rad,ang,winb,wine,hdrs,xy_unit,sx_key,sy_key,rx_key,ry_key,t0,v,dt,dtau,nti)
    do jj=1,n2
      ! Guessed traveltime
      rad(jj) =  sqrt( (hdrs(rx_key,jj)-hdrs(sx_key,jj))**2. + (hdrs(ry_key,jj)-hdrs(sy_key,jj))**2. )*xy_unit
      ang(jj) = atan2( (hdrs(ry_key,jj)-hdrs(sy_key,jj))     , (hdrs(rx_key,jj)-hdrs(sx_key,jj))     )
    !  if (ang.gt. 4.) then;  write(0,*) 'detected'; ang=0;  end if
    !  if (ang.lt.-4.) then;  write(0,*) 'detected'; ang=0;  end if
      
      t0(jj) = nint( rad(jj)/real(v*dt) + 1. )  ! Index of t0
      
      ! Determine indices of window of interest
      winb(jj) = minval((/maxval((/t0(jj)-dtaui,  1/)) , nti/))
      wine(jj) = maxval((/minval((/t0(jj)+dtaui,nti/)) ,   1/))
    end do
!    !$OMP END PARALLEL DO


!    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(n2,data_neg,winb,wine,nti,sn_neg,tindn)
    do jj=1,n2

      ! Estimate S/N ratio in negative time window
      sig_in  = maxval(data_neg_or(winb(jj):wine(jj),jj))
      sig_out = 0.
      if (winb(jj).gt.  1) then
        sig_out = sig_out + sum(abs( data_neg_or(     1:winb(jj)-1,jj) )) / (winb(jj)-   1)
      end if
      if (wine(jj).lt.nti) then
        sig_out = sig_out + sum(abs( data_neg_or(wine(jj)+1:nti   ,jj) )) / (nti -wine(jj))
      end if
      if (sig_out.ne.0.) then
        sn_neg(jj) = sig_in/sig_out
      else
        sn_neg(jj) = -1
      end if
      
      ! Pick maximum
      tindn(jj) = maxval(winb(jj)-1+maxloc(data_neg(winb(jj):wine(jj),jj)))
      
    end do
!    !$OMP END PARALLEL DO

    
!    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(n2,data_pos,winb,wine,nti,sn_pos,tindp)
    do jj=1,n2
      
      ! Estimate S/N ratio in positive time window
      sig_in  = maxval(abs( data_pos_or(winb(jj):wine(jj),jj) ))
      sig_out = 0.
      if (winb(jj).gt.  1) then
        sig_out = sig_out + sum(abs( data_pos_or(     1:winb(jj)-1,jj) )) / (winb(jj)-   1)
      end if
      if (wine(jj).lt.nti) then
        sig_out = sig_out + sum(abs( data_pos_or(wine(jj)+1:nti   ,jj) )) / (nti -wine(jj))
      end if
      if (sig_out.ne.0.) then
        sn_pos(jj) = sig_in/sig_out
      else
        sn_pos(jj) = -1
      end if
            
      ! Pick maximum
      tindp(jj) = maxval(winb(jj)-1+maxloc(data_pos(winb(jj):wine(jj),jj)))
      
    end do
!    !$OMP END PARALLEL DO

!    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(n2,pickdiff,rad,tindp,tindn,dt)
    do jj=1,n2
      ! Store difference
      if (rad(jj).ne.0.) pickdiff(jj) = (real(tindp(jj)-tindn(jj))*dt)/rad(jj)
    end do
!    !$OMP END PARALLEL DO
    
    ! Symmetrize
    data_pos(1,:)      =   datae(nti+1,:)
    do ii=1,nti-1
      data_pos(1+ii,:) = ( datae(nti+1+ii,:) + datae(nti+1-ii,:) ) / 2.
    end do

    data_pos_or(1,:)      =   datae_or(nti+1,:)
    do ii=1,nti-1
      data_pos_or(1+ii,:) = ( datae_or(nti+1+ii,:) + datae_or(nti+1-ii,:) ) / 2.
    end do
    
    
!    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(n2,data_pos,winb,wine,nti,sn_pos,tind,snr)
    do jj=1,n2

      ! Estimate S/N ratio in symmetrized time window
      sig_in  = maxval(abs( data_pos_or(winb(jj):wine(jj),jj) ))
      sig_out = 0.
      
      if (winb(jj).gt.  1) then
        sig_out = sig_out + sum(abs( data_pos_or(     1:winb(jj)-1,jj) )) / (winb(jj)-   1)
      end if
      if (wine(jj).lt.nti) then
        sig_out = sig_out + sum(abs( data_pos_or(wine(jj)+1:nti   ,jj) )) / (nti -wine(jj))
      end if
      if (sig_out.ne.0.) then
        snr(jj) = sig_in/sig_out
      else
        snr(jj) = -1
      end if      
      
      ! Pick maximum
      tind(jj)  = maxval(winb(jj)-1+maxloc(data_pos(winb(jj):wine(jj),jj)))
      
    end do
!    !$OMP END PARALLEL DO

 !   tindn = 0.
 !   tindp = 0.
 !   sn_neg = 0.! snr
 !   sn_pos = 0. !snr
    
    
    do jj=1,n2
      ! Search lower end
      peaklo=tind(jj)
      do ii=tind(jj),1,-1
        if ( data_pos(ii,jj) .gt. .95*data_pos(tind(jj),jj) ) then
           peaklo=ii
        else
           exit
        end if
      end do
      
      ! Search upper end
      peakup=tind(jj)
      do ii=tind(jj),nti,1
        if ( data_pos(ii,jj) .gt. .95*data_pos(tind(jj),jj) ) then
           peakup=ii
        else
           exit
        end if
      end do
      
      peakwidth(jj) = tt(peakup) - tt(peaklo) + dt
    end do

!    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(n2,thresh,datat,values)
!    !$OMP END PARALLEL DO
    
    do jj=1,n2
      ! Construct the output
      values(1 ,jj) = real( tind(jj)-1)*dt       ! T
      values(2 ,jj) = snr(jj)                   ! SNR
      values(3 ,jj) = hdrs(rx_key,jj)*xy_unit  ! Rx
      values(4 ,jj) = hdrs(ry_key,jj)*xy_unit  ! Ry
      values(5 ,jj) = hdrs(sx_key,jj)*xy_unit  ! Sx
      values(6 ,jj) = hdrs(sy_key,jj)*xy_unit  ! Sy
      values(7 ,jj) = rad(jj)                   ! Radius
      values(8 ,jj) = ang(jj)                   ! Azumuth
      values(9 ,jj) = real( winb(jj)-1)*dt       ! Lower limit of picking window
      values(10,jj) = real( wine(jj)-1)*dt       ! Upper limit of picking window
      values(11,jj) =-real(tindn(jj)-1)*dt      ! Negative pick
      values(12,jj) = sn_neg(jj)                ! Negative SNR
      values(13,jj) =-real( winb(jj)-1)*dt       ! Lower limit of negative picking window
      values(14,jj) =-real( wine(jj)-1)*dt       ! Upper limit of negative picking window
      values(15,jj) = real(tindp(jj)-1)*dt      ! Positive pick
      values(16,jj) = sn_pos(jj)                ! Positive SNR
      values(17,jj) = real( winb(jj)-1)*dt       ! Lower limit of positive picking window
      values(18,jj) = real( wine(jj)-1)*dt       ! Upper limit of positive picking window
      values(19,jj) = jj                    ! n2 index
      values(20,jj) = kk                    ! n3 index
      values(21,jj) = peakwidth(jj)            ! n3 index
    end do
    if (extend) values(21+1:21+n1h,:) = hdrs

    ! Check for very weak data
    if (thresh.ne.-1) then
      do jj=1,n2
        if (sum(abs(datat(:,jj))).lt.thresh) then
          values( 2 ,jj) = -1
          values(12 ,jj) = -1
          values(16 ,jj) = -1
        end if
      end do
    end if
    
    ! Extend with headers
    if (exist_file("pickdiff")) call srite("pickdiff",pickdiff,4*n2)
    
    ! Write the SNR
    call srite("out",values,4*size(values))
    
  end do
  call stop_timer_num (tm)
  call print_timers()

  if (.not.extend) call to_history("n1",21    )!,'picks')
  if (     extend) call to_history("n1",21+n1h)!,'picks')
  call to_history("d1",1. )!,'picks')
  call to_history("o1",1. )!,'picks')
  call to_history("n2",n2 )!,'picks')
  call to_history("d2",1. )!,'picks')
  call to_history("o2",1. )!,'picks')
  call to_history("n3",n3 )
  call to_history("n4",1  )
  call to_history("n5",1  )
  call to_history("n6",1  )
  
  
  ! Close all I/O to headerfiles
  call sep_close ()
  
  ! Deallocate Memory
  if (verbose) write(0,*) '-------------------------------------------'
  call sfftw_destroy_plan(fft_for,fft_inv)
  
  deallocate( datat   )
  deallocate( datatc  )
  deallocate( datafc  )
  deallocate( datafac )
  deallocate( dataac  )
  deallocate( datae   )
  
  deallocate( taper   )
  deallocate( hdrs    )
  
  deallocate( data_neg )
  deallocate( data_pos )

  deallocate(data_dft1, dafa_dft1)
  deallocate(data_dft2, dafa_dft2)
  
  deallocate( pickdiff )
  
  deallocate(tt)

  deallocate(rad,ang,t0,winb,wine)
  deallocate(tindn,tindp,tind,sn_pos,sn_neg,snr)
  
  deallocate(peakwidth)

  deallocate(datab,datb)
  deallocate(dataf)

  call exit(0)
  
end program
